import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
/**
 * Write a description of class Pencil here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Pencil extends Actor
{
    private float pWidth;
    private int alpha;
    private GreenfootImage currentLayer;
    private GreenfootImage savedLayer;
    private GreenfootImage texture;
    private IntegerPoint lastLocation;
    /**
     * Act - do whatever the Pencil wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        greenfoot.MouseInfo mouse = Greenfoot.getMouseInfo();
        if (mouse==null) {
            return;
        }
        if (Greenfoot.mousePressed(this)) {
            lastLocation = new IntegerPoint(mouse.getX()+texture.getWidth()/2-getX(),mouse.getY()+texture.getHeight()/2-getY());
            drawLine(lastLocation, lastLocation, mouse.getButton());
        }
        if (Greenfoot.mouseDragged(this)) {
            IntegerPoint newLocation = new IntegerPoint(mouse.getX()+texture.getWidth()/2-getX(),mouse.getY()+texture.getHeight()/2-getY());
            drawLine(lastLocation, newLocation, mouse.getButton());
            lastLocation = newLocation;
        }
    }
    
    public Pencil(GreenfootImage texture, float pWidth, World world)
    {
        this(texture, pWidth, world.getWidth(), world.getHeight());
    }
    
    public Pencil(GreenfootImage texture, float pWidth, int width, int height)
    {
        this.texture = texture;
        setAlpha(255);
        setWidth(pWidth);
        setup(width, height);
    }
    
    private void setup(int width, int height)
    {
        currentLayer = new GreenfootImage(width, height);
        savedLayer = new GreenfootImage(width, height);
        setImage(new GreenfootImage(width, height));
        setupTexture();
    }
    
    private void setupTexture()
    {
        GreenfootImage image = new GreenfootImage(savedLayer.getWidth(), savedLayer.getHeight());
        for (int x = 0; x < image.getWidth(); x+=texture.getWidth()) {
            for (int y = 0; y < image.getHeight(); y+=texture.getHeight()) {
                image.drawImage(texture,x,y);
            }
        }
        texture = image;
    }
    
    public void setTexture(GreenfootImage texture)
    {
        this.texture = texture;
        setupTexture();
        savedLayer = new GreenfootImage(getImage());
        currentLayer.clear();
        updateImage();
    }
    
    public void setTexture(String fileName)
    {
        setTexture(new GreenfootImage(fileName));
    }
    
    public void setWidth(float pWidth)
    {
        this.pWidth = pWidth;
    }
    
    public void setAlpha(int alpha)
    {
        this.alpha = alpha;
    }
    
    public void drawLine(IntegerPoint aP, IntegerPoint bP, int mouseButton)
    {
        if (mouseButton == 1) {
            drawLine(aP, bP);
        }
        else if (mouseButton == 3) {
            eraseLine(aP, bP);
        }
    }
    
    private void drawLine(IntegerPoint aP, IntegerPoint bP)
    {
        Graphics2D graphics = currentLayer.getAwtImage().createGraphics();
        graphics.setComposite(AlphaComposite.Src);
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        graphics.setColor(new Color(0,0,0,alpha));
        graphics.setStroke(new BasicStroke(pWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        graphics.drawLine(aP.getX(), aP.getY(), bP.getX(), bP.getY());
        graphics.setComposite(AlphaComposite.SrcIn);
        graphics.drawImage(texture.getAwtImage(), null, 0, 0);
        graphics.dispose();
        
        updateImage();
    }
    
    private void eraseLine(IntegerPoint aP, IntegerPoint bP)
    {
        Graphics2D graphics = currentLayer.getAwtImage().createGraphics();
        graphics.setComposite(AlphaComposite.Src);
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        graphics.setColor(new Color(0,0,0,0));
        graphics.setStroke(new BasicStroke(pWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        graphics.drawLine(aP.getX(), aP.getY(), bP.getX(), bP.getY());
        graphics.dispose();
        
        graphics = savedLayer.getAwtImage().createGraphics();
        graphics.setComposite(AlphaComposite.Src);
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        graphics.setColor(new Color(0,0,0,0));
        graphics.setStroke(new BasicStroke(pWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        graphics.drawLine(aP.getX(), aP.getY(), bP.getX(), bP.getY());
        graphics.dispose();
        
        updateImage();
    }
    
    private void updateImage()
    {
        Graphics2D graphics = getImage().getAwtImage().createGraphics();
        getImage().clear();
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        graphics.drawImage(savedLayer.getAwtImage(), null, 0, 0);
        graphics.drawImage(currentLayer.getAwtImage(), null, 0, 0);
        graphics.dispose();
    }
}
